' BOINC script v2.0
' 2005-05-27
' John Munro
' mrwizard@wizardstower.co.uk
'
' Changes in v2.0:
' * changed the code that gets the install directory from the registry to search the installed programmes for BOINC since in the newer versions of BOINC they've removed the previous reg key
' * put in detection of single-cpu to avoid problem when option is set to keep paused WU's in memory so they sit in slots other than 0
' * removed "Public" from the signatures of public methods as it caused the Samurize Config Editor to have a parsing error when getting the name of the parameter
' * modified all relevant functions to include cpu id for multi-processor or hyperthreading machines
'
' Changes in v1.7:
' * added getReadyToReportWorkUnitCount
'
' Changes in v1.6:
' * added getTotalCredit
'
' Changes in v1.5:
' * added getCurrentProjectURL, getCurrentResultName, getCurrentProjectUserName and getCurrentProjectTeamName
'
' Changes in v1.4:
' * consolidated repeated code into private functions - script smaller
'
' Changes in v1.3:
' * added getCurrentProjectTotalCredit and getCurrentProjectAverageCredit
'
' Changes in v1.2:
' * improved getCurrentProjectName XPath query - now only makes one query to get the project name
' * changed getDOMDocument return value class from "Msxml2.DOMDocument.3.0" to "Microsoft.XMLDOM" - better not to tie it to any particular version of the class.
'
' Changes in v1.1:
' * get location of state file from registry rather than constant

Option Explicit

Function getCurrentProjectName(cpu)
	On Error Resume Next
	
	getCurrentProjectName = getPropertyOfCurrentProject("project_name", cpu)
End Function

Function getCurrentProjectURL(cpu)
	On Error Resume Next
	
	getCurrentProjectURL = getPropertyOfActiveTask("project_master_url", cpu)
End Function

Function getCurrentProjectUserName(cpu)
	On Error Resume Next
	
	getCurrentProjectUserName = getPropertyOfCurrentProject("user_name", cpu)
End Function

Function getCurrentProjectTeamName(cpu)
	On Error Resume Next
	
	getCurrentProjectTeamName = getPropertyOfCurrentProject("team_name", cpu)
End Function

Function getCurrentResultName(cpu)
	On Error Resume Next
	
	getCurrentResultName = getPropertyOfActiveTask("result_name", cpu)
End Function

Function getCurrentProjectPercent(cpu)
	On Error Resume Next
	
	dim fractionDone
	fractionDone = CDbl(getPropertyOfActiveTask("fraction_done", cpu))
	
	getCurrentProjectPercent = fractionDone * 100
End Function

Function getCurrentProjectElapsedTime(cpu)
	On Error Resume Next
	
	dim seconds
	seconds = CDbl(getPropertyOfActiveTask("current_cpu_time", cpu))

	getCurrentProjectElapsedTime = FormatTime(seconds)
End Function

Function getCurrentProjectRemainingTime(cpu)
	On Error Resume Next

	Dim fractionDone, secondsElapsed, secondsRemaining
	
	secondsElapsed = CDbl(getPropertyOfActiveTask("current_cpu_time", cpu))
	fractionDone = CDbl(getPropertyOfActiveTask("fraction_done", cpu))
	
	if fractionDone <> 0 then
		secondsRemaining = (secondsElapsed / fractionDone) - secondsElapsed
	end if
	
	getCurrentProjectRemainingTime = FormatTime(secondsRemaining)
End Function

Function getCurrentProjectTotalCredit(cpu)
	On Error Resume Next
	
	getCurrentProjectTotalCredit = CDbl(getPropertyOfCurrentProject("user_total_credit", cpu))
End Function

Function getCurrentProjectAverageCredit(cpu)
	On Error Resume Next
	
	getCurrentProjectAverageCredit = CDbl(getPropertyOfCurrentProject("user_expavg_credit", cpu))
End Function

Function getTotalCredit()
	On Error Resume Next
	
	getTotalCredit = 0
	
	Dim xmlDoc
	Set xmlDoc = getDOMDocument
	
	If xmlDoc Is Nothing Then
		Exit Function
	End If
	
	Dim nodes
	Set nodes = xmlDoc.selectNodes("/client_state/project")
	
	dim node
	for each node in nodes
		getTotalCredit = getTotalCredit + node.selectSingleNode("user_total_credit").Text
	next
End Function

Function getReadyToReportWorkUnitCount()
	getReadyToReportWorkUnitCount = 0

	On Error Resume Next
	
	Dim xmlDoc
	Set xmlDoc = getDOMDocument
	
	If xmlDoc Is Nothing Then
		Exit Function
	End If
	
	Dim nodes_results
	set nodes_results = xmlDoc.selectNodes("/client_state/result[state = 5]")
	
	dim iCount, node_result
	for each node_result in nodes_results
		iCount = iCount  + 1
	next

	getReadyToReportWorkUnitCount = iCount
End Function

' PRIVATE FUNCTIONS ----------------------------------------------------------

Private Function getPropertyOfCurrentProject(propName, slot)
	getPropertyOfCurrentProject = getXMLProperty("/client_state/project[master_url = /client_state/active_task_set/active_task[scheduler_state = 2][slot = " & slot & " or /client_state/host_info/p_ncpus = 1]/project_master_url]/" & propName)
End Function

Private Function getPropertyOfActiveTask(propName, slot)
	getPropertyOfActiveTask = getXMLProperty("/client_state/active_task_set/active_task[scheduler_state = 2][slot = " & slot & " or /client_state/host_info/p_ncpus = 1]/" & propName)
End Function

Private Function getXMLProperty(xPathQuery)
	Dim xmlDoc
	Set xmlDoc = getDOMDocument

	If xmlDoc Is Nothing Then
		Exit Function
	End If

	Dim node
	Set node = xmlDoc.selectSingleNode(xPathQuery)

	If Not node Is Nothing Then
		getXMLProperty = node.Text

		Set node = Nothing
	End If

	Set xmlDoc = Nothing
End Function

Private Function getBOINCInstallPath
	const HKEY_LOCAL_MACHINE = &H80000002
	const KEY = "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall"

	dim objReg, arrSubKeys, subkey, sVal

	Set objReg = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\default:StdRegProv")
	
	objReg.EnumKey HKEY_LOCAL_MACHINE, KEY, arrSubKeys

	For Each subkey In arrSubKeys
		objReg.GetStringValue HKEY_LOCAL_MACHINE, KEY & "\" & subkey, "DisplayName", sVal

		if sVal = "BOINC" then
			objReg.GetStringValue HKEY_LOCAL_MACHINE, KEY & "\" & subkey, "InstallLocation", getBOINCInstallPath
			
			exit for
		end if
	Next

	set objReg = nothing
End Function

Private Function getDOMDocument()
	dim fileName
	fileName = getBOINCInstallPath & "\client_state.xml"
	
	Dim FSO
	Set FSO = CreateObject("Scripting.FileSystemObject")
	
	If FSO.FileExists(fileName) then
		Dim TextStream
		Set TextStream = FSO.OpenTextFile(fileName, 1)
		
		Dim sXML
		sXML = TextStream.ReadAll
		TextStream.close

		Set TextStream = Nothing
		Set FSO = Nothing
		
		Dim xmlDoc
		Set xmlDoc = CreateObject("Microsoft.XMLDOM")
		xmlDoc.loadXML sXML
		
		Set getDOMDocument = xmlDoc
	End If
End Function

Private Function FormatTime(seconds)
	Dim hours, minutes

	hours = seconds \ 3600
	seconds = seconds - (hours * 3600)
	minutes = seconds \ 60
	seconds= seconds - (minutes * 60)
	seconds = cint(seconds)
	
	FormatTime = EnsureLengthWithLeading(hours, 2, "0") & ":" & EnsureLengthWithLeading(minutes, 2, "0") & ":" & EnsureLengthWithLeading(seconds, 2, "0")
End Function

Private Function EnsureLengthWithLeading(Value, TargetLength, PadWith)
	if len(Value) < TargetLength then
		EnsureLengthWithLeading = String(TargetLength - len(Value), PadWith) & Value
	else
		EnsureLengthWithLeading = Value
	end if
End Function